// used for rotations
uniform vec2 u_mySpriteRotation;

// crust textures
uniform sampler2D sticky_noise_texture;
uniform vec2 sticky_noise_texturePixelSize;

uniform sampler2D acid_noise_texture;
uniform vec2 acid_noise_texturePixelSize;

uniform sampler2D green_grass_texture;
uniform vec2 green_grass_texturePixelSize;

uniform vec2 cc_MainTexturePixelSize;

uniform vec4 tint_color;

uniform sampler2D mix_texture;

uniform sampler2D rgba_mask_texture;


// helper functions
vec4 composite(vec4 over, vec4 under){
	return over + (1.0 - over.a)*under;
}
vec4 premultiply(vec4 color){
    return vec4(color.r*color.a, color.g*color.a, color.b*color.a, color.a);
}

void main(){
    
    // Time offset
    highp float offset = cc_Time[0] * 1.0;
    
    // Raw colors
    vec4 transparent_color = vec4(0.0, 0.0, 0.0, 0.0);
    
    // Pink colors
    vec4 pink_color = vec4(1.0, 0.5, 1.0, 1.0);
    vec4 pink_middle_color = vec4(0.92, 0.3, 0.92, 1.0);
    vec4 pink_dark_color = vec4(0.75,  0.25, 0.73, 1.0);
    
    // Acid colors
    vec4 light_green_color = vec4(0.2, 1.0, 0.27, 1.0);
    vec4 mid_green_color = vec4(0.07, 0.87, 0.13, 1.0 );
    vec4 dark_green_color = vec4(0.07, 0.67, 0.13, 1.0);
    
    // Sample the 4 channel mask texture
    vec4 rgbaMaskColor = cc_FragColor*texture2D(rgba_mask_texture, cc_FragTexCoord1);
    // Sampling the sdf
    float sdf_color = rgbaMaskColor.a;
    // Sampling our paint masks
    float sticky_painted_color_mask = rgbaMaskColor.r;
    float acid_painted_color_mask = rgbaMaskColor.g;
    float green_mask_color = rgbaMaskColor.b;
    
    
    
    // Sampling (pre-divide) the main terrain texture and alpha
    vec4 originalTextureColor = cc_FragColor*texture2D(cc_MainTexture, cc_FragTexCoord1);
    float terrain_alpha = originalTextureColor.a;
    
    
    // Tile and sample the repeating crust texture
    // we're assuming sticky and acid noise are the same sizes here...
    float tiledRatioX = cc_MainTexturePixelSize.x / sticky_noise_texturePixelSize.x;
    float tiledRatioY = cc_MainTexturePixelSize.y / sticky_noise_texturePixelSize.y;
    
    // sticky
    vec2 sticky_noise_texcoord = vec2( cc_FragTexCoord1.x*tiledRatioX, cc_FragTexCoord1.y*tiledRatioY );
    vec4 sticky_noise_color = cc_FragColor*texture2D(sticky_noise_texture, vec2(sticky_noise_texcoord.x, sticky_noise_texcoord.y + ((offset*0.25)*0.1)));
    
    
    
    // acid
    vec2 acid_noise_texcoord = vec2( cc_FragTexCoord1.x*tiledRatioX, cc_FragTexCoord1.y*tiledRatioY );
    vec4 acid_noise_color_test = cc_FragColor*texture2D(acid_noise_texture, vec2(acid_noise_texcoord.x + (offset*0.0121), acid_noise_texcoord.y -(offset*0.02738)));
    vec4 acid_noise_color = cc_FragColor*texture2D(acid_noise_texture, vec2(acid_noise_texcoord.x - (offset*0.0489), acid_noise_texcoord.y + (offset*0.01927) ));
    
    acid_noise_color = (acid_noise_color + acid_noise_color_test) / 2.0;
    
    // floats for density, and crust red channel
    highp float density_overlay = sdf_color; // (raw density)
    
    // sticky
    highp float sticky_density = max(0.0, sdf_color + (1.0 - sticky_painted_color_mask));  // (density combined with overall painted mask)
    highp float sticky_crust = sticky_noise_color.r;
    
    // acid
    
    highp float acid_density = max(0.0, sdf_color + (1.0 - acid_painted_color_mask));  // (density combined with overall painted mask)
    highp float acid_crust = acid_noise_color.r;
    
    

    float SmoothCenter2 = 0.51;
    float SmoothWidth2 = 0.01;
    mediump float alpha2 = smoothstep(SmoothCenter2 + SmoothWidth2,
                                      SmoothCenter2 - SmoothWidth2,
                                      density_overlay);
                                      // density);
    
    
    // The mixing and crust lookup textures control the shape of the terrain's crust layer.
    // -------------------------------------------------------
    // -------------------------------------------------------
    // Sticky logic
    // -------------------------------------------------------
    // Dark pink base...
    lowp vec4 mix_color = texture2D(mix_texture, vec2(sticky_density, sticky_crust*0.5));
    //vec4 finalColor = mix(mix(terrain_color, pink_dark_color, mix_color.a), terrain_color, mix_color.r);
    vec4 finalColor = mix(mix(transparent_color, pink_dark_color, mix_color.a), transparent_color, mix_color.r);
    vec4 final_painted_color = finalColor;
    
    
    // Light pink mixing...
    lowp vec4 mix_color_dark = texture2D(mix_texture, vec2(sticky_density, 0.5 + (sticky_crust*0.5)));
    vec4 finalColor_dark = mix(mix(final_painted_color, pink_middle_color, mix_color_dark.a), final_painted_color, mix_color_dark.r);
    final_painted_color = finalColor_dark;
    
    
    // Multiply the mix_color into the overlay so it doesn't outline everything
    vec4 light_pink_outline = vec4(pink_color.rgb*alpha2*(1.0-mix_color.r), alpha2*(1.0-mix_color.r));
    vec4 tmpComposite = composite(light_pink_outline, final_painted_color);
    //vec4 tmpComposite = mix(final_painted_color, light_pink_outline, light_pink_outline.a);
    
    
    
    
    
    // here, now lets try and do the same things with acid....
    // -------------------------------------------------------
    // -------------------------------------------------------
    // Acid logic
    // -------------------------------------------------------
    // Dark green base...
    lowp vec4 acid_mix_color = texture2D(mix_texture, vec2(acid_density, acid_crust*0.5));
    vec4 acid_finalColor = mix(mix(transparent_color, dark_green_color, acid_mix_color.a), transparent_color, acid_mix_color.r);
    vec4 acid_final_painted_color = acid_finalColor;
    
    
    
    // Light green mixing...
    lowp vec4 acid_mix_color_dark = texture2D(mix_texture, vec2(min(1.0,acid_density*1.1), 0.5 + (acid_crust*0.5)));
    vec4 acid_finalColor_dark = mix(mix(acid_final_painted_color, mid_green_color, acid_mix_color_dark.a), acid_final_painted_color, acid_mix_color_dark.r);
    acid_final_painted_color = acid_finalColor_dark;
    
    
    // Multiply the mix_color into the overlay so it doesn't outline everything
    vec4 light_green_outline = vec4(light_green_color.rgb*alpha2*(1.0-acid_mix_color.r), alpha2*(1.0-acid_mix_color.r));
    vec4 acid_tmpComposite = composite(light_green_outline, acid_final_painted_color);
    
    
    
    // this gives us just pink
    //gl_FragColor = vec4(tmpComposite.r*terrain_alpha, tmpComposite.g*terrain_alpha, tmpComposite.b*terrain_alpha, terrain_alpha);
 
    // this give us just acid
    //gl_FragColor = vec4(acid_tmpComposite.r*terrain_alpha, acid_tmpComposite.g*terrain_alpha, acid_tmpComposite.b*terrain_alpha, terrain_alpha);
    
    
    // sticky is composited onto the terrain color, acid is composited onto transparent color.
    // so compositing them ontop of each will work
    vec4 finalComposite = composite(acid_tmpComposite, tmpComposite);
    
    
    // this is final overlay color  // no original texture is included in this final composite...
    
    
    // new stuff below here...
    
    
    vec4 testGreenColor = tint_color;
    vec4 originalOverlaysColor = finalComposite;
    vec4 textureColor = originalTextureColor;
    vec2 shadowOffset = u_mySpriteRotation;

    // TODO:, this ends up oversampling the transparent areas, when all we really need is the transparent areas above
    // the opaque areas.  I think we can further optimize this with the distance field.
    //if(originalTextureColor.a < 1.0) { // conditional logic...  i think this is faster than 10x sampling every fragment.
    if(sdf_color > 0.42 && sdf_color < 0.58) {
        for(int i =1; i < 6; i++) {
            float floatI = float(i);
            vec4 shadowColor = cc_FragColor*texture2D(cc_MainTexture, cc_FragTexCoord1 + (shadowOffset*(floatI/5.0)));
            
            vec4 setColor = vec4(testGreenColor.r*shadowColor.a, testGreenColor.g*shadowColor.a, testGreenColor.b*shadowColor.a, shadowColor.a);
            
            vec4 mixedColor = mix(shadowColor, setColor, testGreenColor.a);
            
            textureColor = composite(textureColor, mixedColor);
        }
    }
    
    originalOverlaysColor = vec4(originalOverlaysColor.r*textureColor.a, originalOverlaysColor.g*textureColor.a, originalOverlaysColor.b*textureColor.a, originalOverlaysColor.a*textureColor.a);
    textureColor = composite(originalOverlaysColor, textureColor);
    
   
    
    // now we sample the green mask,
    float grassTiledRatioX = cc_MainTexturePixelSize.x / green_grass_texturePixelSize.x;
    float grassTiledRatioY = cc_MainTexturePixelSize.y / green_grass_texturePixelSize.y;
    

    
    float green_mask_alpha = green_mask_color;
    vec4 green_tex_color = cc_FragColor*texture2D(green_grass_texture, vec2( cc_FragTexCoord1.x*grassTiledRatioX, cc_FragTexCoord1.y*grassTiledRatioY ));
    
    

    
    vec4 green_color = vec4(green_tex_color.r, green_tex_color.g, green_tex_color.b, green_mask_alpha);
    
    
    //textureColor = predivide(textureColor);
    //    gl_FragColor = textureColor;
    //    return;
    
    //float original_texture_alpha = textureColor.a;
    
    // smooth alpha from sdf....
    SmoothCenter2 = 0.51;
    SmoothWidth2 = 0.005;
    alpha2 = smoothstep(SmoothCenter2 + SmoothWidth2,
                                      SmoothCenter2 - SmoothWidth2,
                                      density_overlay);
    
    
    

    float greenAlpha = alpha2 * green_mask_alpha * textureColor.a;
    green_color = vec4(green_color.r*greenAlpha, green_color.g*greenAlpha, green_color.b*greenAlpha, greenAlpha);
    
    // composite green onto tex.
    vec4 finalCompositeFinal = composite(green_color, textureColor);

    // assign original tex alpha to everthing..
    //finalCompositeFinal = vec4(finalCompositeFinal.r* textureColor.a, finalCompositeFinal.g* textureColor.a, finalCompositeFinal.b* textureColor.a, textureColor.a);
    
    
    //finalCompositeFinal = vec4(finalCompositeFinal.r* textureColor.a, finalCompositeFinal.g* textureColor.a, finalCompositeFinal.b* textureColor.a, textureColor.a);
    
    
    
    
    gl_FragColor = finalCompositeFinal;
    
    
    
    //gl_FragColor = finalComposite;
    
    //gl_FragColor = textureColor;
    //gl_FragColor = finalComposite;
    
    
    
    // here we everything but unmasked....
    
    
    
    
    //gl_FragColor = vec4(finalComposite.r*terrain_alpha, finalComposite.g*terrain_alpha, finalComposite.b*terrain_alpha, terrain_alpha);
  
}




