uniform sampler2D u_NoiseTexture;

vec2 mod289(vec2 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec3 mod289(vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec4 mod289(vec4 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}

vec3 permute(vec3 x) {
    return mod289(((x*34.0)+1.0)*x);
}

vec4 permute(vec4 x) {
    return mod289(((x*34.0)+1.0)*x);
}

vec4 taylorInvSqrt(vec4 r)
{
    return 1.79284291400159 - 0.85373472095314 * r;
}

float snoise2(vec2 v)
{
    const vec4 C = vec4(0.211324865405187,  // (3.0-sqrt(3.0))/6.0
                        0.366025403784439,  // 0.5*(sqrt(3.0)-1.0)
                        -0.577350269189626,  // -1.0 + 2.0 * C.x
                        0.024390243902439); // 1.0 / 41.0
    // First corner
    vec2 i  = floor(v + dot(v, C.yy) );
    vec2 x0 = v -   i + dot(i, C.xx);
    
    // Other corners
    vec2 i1;
    //i1.x = step( x0.y, x0.x ); // x0.x > x0.y ? 1.0 : 0.0
    //i1.y = 1.0 - i1.x;
    i1 = (x0.x > x0.y) ? vec2(1.0, 0.0) : vec2(0.0, 1.0);
    // x0 = x0 - 0.0 + 0.0 * C.xx ;
    // x1 = x0 - i1 + 1.0 * C.xx ;
    // x2 = x0 - 1.0 + 2.0 * C.xx ;
    vec4 x12 = x0.xyxy + C.xxzz;
    x12.xy -= i1;
    
    // Permutations
    i = mod289(i); // Avoid truncation effects in permutation
    vec3 p = permute( permute( i.y + vec3(0.0, i1.y, 1.0 ))
                     + i.x + vec3(0.0, i1.x, 1.0 ));
    
    vec3 m = max(0.5 - vec3(dot(x0,x0), dot(x12.xy,x12.xy), dot(x12.zw,x12.zw)), 0.0);
    m = m*m ;
    m = m*m ;
    
    // Gradients: 41 points uniformly over a line, mapped onto a diamond.
    // The ring size 17*17 = 289 is close to a multiple of 41 (41*7 = 287)
    
    vec3 x = 2.0 * fract(p * C.www) - 1.0;
    vec3 h = abs(x) - 0.5;
    vec3 ox = floor(x + 0.5);
    vec3 a0 = x - ox;
    
    // Normalise gradients implicitly by scaling m
    // Approximation of: m *= inversesqrt( a0*a0 + h*h );
    m *= 1.79284291400159 - 0.85373472095314 * ( a0*a0 + h*h );
    
    // Compute final noise value at P
    vec3 g;
    g.x  = a0.x  * x0.x  + h.x  * x0.y;
    g.yz = a0.yz * x12.xz + h.yz * x12.yw;
    return 130.0 * dot(m, g);
}




void main() {
    
    highp vec2 position = cc_Position.xy;
    /*
    vec2 normal = cc_TexCoord2 *0.1; // this coming in...
    cc_FragColor = clamp(cc_Color, 0.0, 1.0); //
    */
    highp float time = cc_Time[0];
    
    /* this does global waves */
    /*
    vec4 absPosition = cc_ProjectionInv*cc_Position;
    highp float noise = snoise(vec3(cc_Position.x*1.0, cc_Position.y*1.0, time));
     vec2 newPos = position + (noise * 1.0);
     gl_Position = vec4(cc_Position.x, newPos.y, cc_Position.z, cc_Position.w);
     */
    
    /* this does per vertex waves */
    //vec4 absPosition = cc_ProjectionInv*cc_Position;
    //highp float noise = snoise3(vec3(cc_TexCoord1.x, cc_TexCoord2.x, time));
    //vec2 newPos = position + (noise * 0.02);
    //gl_Position = vec4(newPos.x, newPos.y, cc_Position.z, cc_Position.w);
    
    
    //vec4 noiseColor = texture2D(u_NoiseTexture, vec2(cc_TexCoord1.x + cc_TexCoord2.x, time));
    //vec4 noiseColor = texture2D(u_NoiseTexture, vec2(cc_TexCoord1.x, time));
    //vec2 newPos = position + (noiseColor.x * 0.02);
    //gl_Position = vec4(newPos.x, newPos.y, cc_Position.z, cc_Position.w);
    
    
    //gl_Position = vec4(cc_Position.x+((noiseColor.x-0.5)*0.1), cc_Position.y+((noiseColor.x-0.5)*0.1), cc_Position.z, cc_Position.w);
    //vec4 noiseColor = texture2D(u_NoiseTexture, vec2(cc_TexCoord2.x, time));
    //gl_Position = vec4(cc_Position.x+(noiseColor.x*10.0), cc_Position.y+(noiseColor.y*10.0), cc_Position.z, cc_Position.w);
    
    
    
    
    
    highp float noise1 = texture2D(u_NoiseTexture, vec2(cc_TexCoord1.y*cc_TexCoord2.x*0.5, (time*0.5))).x - 0.5;
    highp float noise2 = texture2D(u_NoiseTexture, vec2(cc_TexCoord1.y*cc_TexCoord2.x*0.5, (time*0.5)+0.238)).x - 0.5;
    
    //highp float noise = snoise2(vec2(cc_TexCoord1.x + cc_TexCoord2.x, time));
    highp vec2 newPos = vec2(position.x + (noise1 * 0.04), position.y + (noise2 * 0.04));
    gl_Position = vec4(newPos.x, newPos.y, cc_Position.z, cc_Position.w);

    
    cc_FragColor = clamp(cc_Color, 0.0, 1.0);
    cc_FragTexCoord1 = cc_TexCoord1;
    cc_FragTexCoord2 = cc_TexCoord2;
}
